@@ -182,6 +182,8 @@ import_from_file (NMVpnEditorPlugin *self,
 			nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_RIGHT, &str[6]);
 		else if (g_str_has_prefix (str, "leftxauthusername="))
 			nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTXAUTHUSER, &str[18]);
+		else if (g_str_has_prefix (str, "leftusername="))
+			nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTUSERNAME, &str[13]);
 		else if (g_str_has_prefix (str, "leftcert="))
 			nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTCERT, &str[9]);
 		else if (g_str_has_prefix (str, "pfs=no"))
@@ -268,6 +270,8 @@ export_to_file (NMVpnEditorPlugin *self,
 	gboolean openswan = FALSE;
 	int fd, errsv;
 	gs_free_error GError *local = NULL;
+	gboolean is_openswan;
+	int version;
 
 	fd = g_open (path, O_WRONLY | O_CREAT, 0666);
 	if (fd == -1) {
@@ -281,7 +285,11 @@ export_to_file (NMVpnEditorPlugin *self,
 	if (s_vpn)
 		openswan = nm_streq (nm_setting_vpn_get_service_type (s_vpn), NM_VPN_SERVICE_TYPE_OPENSWAN);
 
+	nm_libreswan_detect_version (nm_libreswan_find_helper_bin ("ipsec", NULL),
+	                             &is_openswan, &version, NULL);
+
 	if (!nm_libreswan_config_write (fd,
+	                                version,
 	                                connection,
 	                                nm_connection_get_id (connection),
 	                                NULL,
@@ -324,23 +332,35 @@ _call_editor_factory (gpointer factory,
 static NMVpnEditor *
 get_editor (NMVpnEditorPlugin *iface, NMConnection *connection, GError **error)
 {
+	gpointer gtk3_only_symbol;
+	GModule *self_module;
+	const char *editor;
+
 	g_return_val_if_fail (LIBRESWAN_IS_EDITOR_PLUGIN (iface), NULL);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
-	{
-#ifdef NM_VPN_OLD
-		return nm_vpn_editor_new (connection, error);
-#else
-		return nm_vpn_plugin_utils_load_editor (NM_PLUGIN_DIR"/libnm-vpn-plugin-libreswan-editor.so",
-		                                        "nm_vpn_editor_factory_libreswan",
-		                                        _call_editor_factory,
-		                                        iface,
-		                                        connection,
-		                                        NULL,
-		                                        error);
-#endif
+        self_module = g_module_open (NULL, 0);
+        g_module_symbol (self_module, "gtk_container_add", &gtk3_only_symbol);
+        g_module_close (self_module);
+
+	if (gtk3_only_symbol) {
+		editor = "libnm-vpn-plugin-libreswan-editor.so";
+	} else {
+		editor = "libnm-gtk4-vpn-plugin-libreswan-editor.so";
 	}
+
+#ifdef NM_VPN_OLD
+	return nm_vpn_editor_new (connection, error);
+#else
+	return nm_vpn_plugin_utils_load_editor (editor,
+						"nm_vpn_editor_factory_libreswan",
+						_call_editor_factory,
+						iface,
+						connection,
+						NULL,
+						error);
+#endif
 }
 
 static void
