@@ -89,6 +89,7 @@ write_config_option_newline (int fd,
 
 gboolean
 nm_libreswan_config_write (gint fd,
+                           int ipsec_version,
                            NMConnection *connection,
                            const char *con_name,
                            const char *leftupdown_script,
@@ -116,8 +117,8 @@ nm_libreswan_config_write (gint fd,
 	const char *rekey;
 	const char *fragmentation;
 	const char *mobike;
+	const char *pfs;
 	gboolean is_ikev2 = FALSE;
-	gboolean xauth_enabled = TRUE;
 
 	g_return_val_if_fail (fd > 0, FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -128,10 +129,9 @@ nm_libreswan_config_write (gint fd,
 	g_return_val_if_fail (NM_IS_SETTING_VPN (s_vpn), FALSE);
 
 	is_ikev2 = nm_libreswan_utils_setting_is_ikev2 (s_vpn, &ikev2);
-	/* When IKEv1 is in place, we enforce XAUTH */
-	xauth_enabled = !is_ikev2;
+
 	/* When using IKEv1 (default in our plugin), we should ensure that we make
-	 * it explicit to Libreswan (which defaults to IKEv2): when crypto algorithms
+	 * it explicit to Libreswan (which now defaults to IKEv2): when crypto algorithms
 	 * are not specified ("esp" & "ike") Libreswan will use system-wide crypto
 	 * policies based on the IKE version in place.
 	 */
@@ -149,7 +149,7 @@ nm_libreswan_config_write (gint fd,
 
 	WRITE_CHECK (fd, debug_write_fcn, error, "conn %s", con_name);
 	if (leftid && strlen (leftid)) {
-		if (xauth_enabled)
+		if (!is_ikev2)
 			WRITE_CHECK (fd, debug_write_fcn, error, " aggrmode=yes");
 
 		if (   leftid[0] == '%'
@@ -209,17 +209,28 @@ nm_libreswan_config_write (gint fd,
 	else
 		WRITE_CHECK (fd, debug_write_fcn, error, " rightsubnet=%s",
 			     remote_network);
-	if (xauth_enabled) {
+
+	if (!is_ikev2) {
+		/* When IKEv1 is in place, we enforce XAUTH: so, use IKE version
+		 * also to check if XAUTH conf options should be passed to Libreswan.
+		 */
 		WRITE_CHECK (fd, debug_write_fcn, error, " leftxauthclient=yes");
 
 		default_username = nm_setting_vpn_get_user_name (s_vpn);
 		props_username = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTXAUTHUSER);
+		if (!props_username)
+			props_username = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTUSERNAME);
 		if (props_username && strlen (props_username))
-			WRITE_CHECK (fd, debug_write_fcn, error, " leftxauthusername=%s", props_username);
+			WRITE_CHECK (fd, debug_write_fcn, error,
+			             ipsec_version >= 4 ? " leftusername=%s" : " leftxauthusername=%s",
+			             props_username);
 		else if (default_username && strlen (default_username))
-			WRITE_CHECK (fd, debug_write_fcn, error, " leftxauthusername=%s", default_username);
+			WRITE_CHECK (fd, debug_write_fcn, error,
+			             ipsec_version >= 4 ? " leftusername=%s" : " leftxauthusername=%s",
+			             default_username);
 
-		WRITE_CHECK (fd, debug_write_fcn, error, " remote_peer_type=cisco");
+		WRITE_CHECK (fd, debug_write_fcn, error,
+		             ipsec_version >= 4 ? " remote-peer-type=cisco" : " remote_peer_type=cisco");
 		WRITE_CHECK (fd, debug_write_fcn, error, " rightxauthserver=yes");
 	}
 
@@ -235,28 +246,32 @@ nm_libreswan_config_write (gint fd,
 	 */
 	if (phase1_alg_str && strlen (phase1_alg_str))
 		WRITE_CHECK (fd, debug_write_fcn, error, " ike=%s", phase1_alg_str);
-	else if (xauth_enabled && leftid)
-		WRITE_CHECK (fd, debug_write_fcn, error, " ike=aes256-sha1;modp1536");
+	else if (!is_ikev2 && leftid)
+		WRITE_CHECK (fd, debug_write_fcn, error, " ike=%s", NM_LIBRESWAN_AGGRMODE_DEFAULT_IKE);
 
 	phase2_alg_str = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_ESP);
 	if (phase2_alg_str && strlen (phase2_alg_str))
 		WRITE_CHECK (fd, debug_write_fcn, error, " phase2alg=%s", phase2_alg_str);
-	else if (xauth_enabled && leftid)
-		WRITE_CHECK (fd, debug_write_fcn, error, " phase2alg=aes256-sha1");
+	else if (!is_ikev2 && leftid)
+		WRITE_CHECK (fd, debug_write_fcn, error, " phase2alg=%s", NM_LIBRESWAN_AGGRMODE_DEFAULT_ESP);
+
+	pfs = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_PFS);
+	if (pfs && !strcmp (pfs, "no"))
+		WRITE_CHECK (fd, debug_write_fcn, error, " pfs=no");
 
 	phase1_lifetime_str = nm_setting_vpn_get_data_item (s_vpn,
 							    NM_LIBRESWAN_KEY_IKELIFETIME);
 	if (phase1_lifetime_str && strlen (phase1_lifetime_str))
 		WRITE_CHECK (fd, debug_write_fcn, error, " ikelifetime=%s", phase1_lifetime_str);
 	else if (!is_ikev2)
-		WRITE_CHECK (fd, debug_write_fcn, error, " ikelifetime=24h");
+		WRITE_CHECK (fd, debug_write_fcn, error, " ikelifetime=%s", NM_LIBRESWAN_IKEV1_DEFAULT_LIFETIME);
 
 	phase2_lifetime_str = nm_setting_vpn_get_data_item (s_vpn,
 							    NM_LIBRESWAN_KEY_SALIFETIME);
 	if (phase2_lifetime_str && strlen (phase2_lifetime_str))
 		WRITE_CHECK (fd, debug_write_fcn, error, " salifetime=%s", phase2_lifetime_str);
 	else if (!is_ikev2)
-		WRITE_CHECK (fd, debug_write_fcn, error, " salifetime=24h");
+		WRITE_CHECK (fd, debug_write_fcn, error, " salifetime=%s", NM_LIBRESWAN_IKEV1_DEFAULT_LIFETIME);
 
 	rekey = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_REKEY);
 	if (!rekey || !strlen (rekey)) {
@@ -288,3 +303,119 @@ nm_libreswan_config_write (gint fd,
 
 	return TRUE;
 }
+
+static const char *
+_find_helper (const char *progname, const char **paths, GError **error)
+{
+	const char **iter = paths;
+	GString *tmp;
+	const char *ret = NULL;
+
+	if (error)
+		g_return_val_if_fail (*error == NULL, NULL);
+
+	tmp = g_string_sized_new (50);
+	for (iter = paths; iter && *iter; iter++) {
+		g_string_append_printf (tmp, "%s%s", *iter, progname);
+		if (g_file_test (tmp->str, G_FILE_TEST_EXISTS)) {
+			ret = g_intern_string (tmp->str);
+			break;
+		}
+		g_string_set_size (tmp, 0);
+	}
+	g_string_free (tmp, TRUE);
+
+	if (!ret) {
+		g_set_error (error, NM_VPN_PLUGIN_ERROR, NM_VPN_PLUGIN_ERROR_LAUNCH_FAILED,
+		             "Could not find %s binary",
+		             progname);
+	}
+	return ret;
+}
+
+const char *
+nm_libreswan_find_helper_bin (const char *progname, GError **error)
+{
+	static const char *paths[] = {
+		PREFIX "/sbin/",
+		PREFIX "/bin/",
+		"/sbin/",
+		"/usr/sbin/",
+		"/usr/local/sbin/",
+		"/usr/bin/",
+		"/usr/local/bin/",
+		NULL,
+	};
+
+	return _find_helper (progname, paths, error);
+}
+
+const char *
+nm_libreswan_find_helper_libexec (const char *progname, GError **error)
+{
+	static const char *paths[] = {
+		PREFIX "/libexec/ipsec/",
+		PREFIX "/lib/ipsec/",
+		"/usr/libexec/ipsec/",
+		"/usr/local/libexec/ipsec/",
+		"/usr/lib/ipsec/",
+		"/usr/local/lib/ipsec/",
+		NULL,
+	};
+
+	return _find_helper (progname, paths, error);
+}
+
+void
+nm_libreswan_detect_version (const char *path, gboolean *out_is_openswan, int *out_version, char **out_banner)
+{
+	const char *argv[] = { path, "--version", NULL };
+	char *output = NULL;
+	const char* v;
+
+	g_return_if_fail (out_is_openswan);
+	g_return_if_fail (out_version);
+
+	*out_is_openswan = FALSE;
+	*out_version = -1;
+
+	if (!path)
+		return;
+
+	g_spawn_sync (NULL, (char **) argv, NULL, 0, NULL, NULL, &output, NULL, NULL, NULL);
+	if (!output)
+		return;
+
+	/*
+	 * Examples:
+	 * Linux Openswan 2.4.5 (klips)
+	 * Linux Libreswan 3.32 (netkey) on 5.8.11-200.fc32.x86_64+debug
+	 * Linux Libreswan U4.2rc1/K(no kernel code presently loaded) on 5.6.15-300.fc32.x86_64
+	 */
+
+	v = strcasestr (output, "Openswan");
+	if (v) {
+		v = v + strlen ("Openswan");
+		*out_is_openswan = TRUE;
+	}
+
+	if (!v) {
+		v = strcasestr (output, "Libreswan");
+		if (v)
+			v = v + strlen ("Libreswan");
+	}
+
+	if (v) {
+		while (g_ascii_isspace (*v))
+			v++;
+		if (*v == 'U')
+			v++;
+		if (g_ascii_isdigit (*v))
+			*out_version = *v - '0';
+	}
+
+	if (out_banner)
+		*out_banner = output;
+	else
+		g_free (output);
+}
